WoWJapanizer = LibStub("AceAddon-3.0"):NewAddon("WoWJapanizer", "AceConsole-3.0")

WoWJapanizer.FONT = "Interface\\AddOns\\WoWJapanizer\\font\\ipagui.ttf";
WoWJapanizer.DEBUG = false

WoWJapanizer.property = {}

function WoWJapanizer:OnInitialize()
    self.version = GetAddOnMetadata("WoWJapanizer", "Version")
    print(string.format("Welcome to WoWJapanizer Ver: %s.\nSetting is /cj or /WoWJapanizer.", self.version))

    self.db = LibStub('AceDB-3.0'):New("WoWJapanizerDB")
	self.db:RegisterDefaults({
        profile = {
            quest       = { questlog = true, worldmap = true, gossip = true, questlog_movable = false },
            item        = { tooltip = true },
            spell       = { tooltip = true, buff = true },
            achievement = { tooltip = true, advice = true },
            developer   = false,
            development = { debugger = false },
        }
    })

    -- Quest --
    WoWJapanizerQuestLog:OnInitialize()
    WoWJapanizerWorldMapQuest:OnInitialize()
    WoWJapanizerQuestGossip:OnInitialize()

    -- ToolTip --
    WoWJapanizerGameToolTip:OnInitialize()
    WoWJapanizerItemRefTooltip:OnInitialize()

    WoWJapanizerBuffToolTip:OnInitialize()
    WoWJapanizerGlyphToolTip:OnInitialize()
end

function WoWJapanizer:OnEnable()
    self:DebugLog("WoWJapanizer: OnEnable.");

    -- Option --
    self:SetupOptions()

    -- Quest --
    WoWJapanizerQuestLog:OnEnable()
    WoWJapanizerWorldMapQuest:OnEnable()
    WoWJapanizerQuestGossip:OnEnable()

    -- ToolTip --
    WoWJapanizerGameToolTip:OnEnable()
    WoWJapanizerItemRefTooltip:OnEnable()

    WoWJapanizerBuffToolTip:OnEnable()
    WoWJapanizerGlyphToolTip:OnEnable()
end

function WoWJapanizer:OnDisable()
    self:DebugLog("WoWJapanizer: OnDisable.");

    -- Quest --
    WoWJapanizerQuestLog:OnDisable()
    WoWJapanizerWorldMapQuest:OnDisable()
    WoWJapanizerQuestGossip:OnDisable()

    -- ToolTip --
    WoWJapanizerGameToolTip:OnDisable()
    WoWJapanizerItemRefTooltip:OnDisable()

    WoWJapanizerBuffToolTip:OnDisable()
    WoWJapanizerGlyphToolTip:OnDisable()
end

function WoWJapanizer:LoadAddOn(addon)
    if not IsAddOnLoaded(addon) then
        if not IsAddOnLoadOnDemand(addon) then
            self:Print("Error: " .. addon .. " Can't find.")
            return false
        end

        local loaded, reason = LoadAddOn(addon)
        if not loaded then
            self:Print("Error: " .. addon .. " Can't find.")
            return false
        end
    end

    return true
end

function WoWJapanizer:uc(str)
    return string.gsub(str, "(%w)", function(s)
        return string.upper(s)
    end, 1)
end

function WoWJapanizer:lc(str)
    return string.gsub(str, "(%w)", function(s)
        return string.lower(s)
    end, 1)
end

function WoWJapanizer:SetupOptions()
    function GetProperty()
        function getDataVersion(name)
            local addon = "WoWJapanizer_" .. self:uc(name)
            if IsAddOnLoadOnDemand(addon) then
                return GetAddOnMetadata(addon, "Version")
            else
                return nil
            end
        end

        return {
            quest = {
                version = getDataVersion("quest")
            },
            item = {
                version = getDataVersion("item")
            },
            spell = {
                version = getDataVersion("spell")
            },
            achievement = {
                version = getDataVersion("achievement")
            },
        }
    end

    function GetOptions()
        function GetDescription(name, text)
            local addon = "WoWJapanizer_" .. self:uc(name)
            if self.property[name].version then
                return string.format('%s version : |cffff9900%s|r\nJapanese translation of %s is displayed.', addon, self.property[name].version, text)
            else
                return string.format('"|cffff0000%s|r" is necessary for Japanese-translated %s. Please get "|cffff0000%s|r" from wow.curse.com.', addon, text, addon)
            end
        end

        return {
            type = "group",
            name = "WoWJapanizer",
            args = {
                general = {
                    type = "group",
                    name = "WoWJapanizer",
                    args = {
                        desc = {
                            type = "description",
                            name = "WoWJapanizer is addon that displays the Japanese translation of Quest information and Spell/Item ToolTips.",
                            order = 1,
                        },
                        quest = {
                            type = "group",
                            name = "Quest",
                            inline = true,
                            order = 2,
                            args = {
                                desc = {
                                    type = "description",
                                    name = GetDescription("quest", "quest"),
                                    order = 1,
                                },
                            },
                        },
                        item = {
                            type = "group",
                            name = "Item ToolTip",
                            inline = true,
                            order = 3,
                            args = {
                                desc = {
                                    type = "description",
                                    name = GetDescription("item", "Item ToolTip"),
                                    order = 1,
                                },
                            },
                        },
                        spell = {
                            type = "group",
                            name = "Spell ToolTip",
                            inline = true,
                            order = 4,
                            args = {
                                desc = {
                                    type = "description",
                                    name = GetDescription("spell", "Spell ToolTip"),
                                    order = 1,
                                },
                            },
                        },
                        achievement = {
                            type = "group",
                            name = "Achievement  |cff9900ff(Beta Test)|r",
                            inline = true,
                            order = 5,
                            args = {
                                desc = {
                                    type = "description",
                                    name = GetDescription("achievement", "Achievement"),
                                    order = 1,
                                },
                            },
                        },
                    },
                },
                quest = {
                    type = "group",
                    name = "WoWJapanizer Quest",
                    args = {
                        desc = {
                            type = "description",
                            name = GetDescription("quest", "quest"),
                            order = 1,
                        },
                        questlog = {
                            type = "group",
                            name = "QuestLog",
                            inline = true,
                            order = 2,
                            args = {
                                desc = {
                                    type = "description",
                                    name = "This setting is display the Japanese translation of QuestLog by default.",
                                    order = 1,
                                },
                                questlog = {
                                    type = "toggle",
                                    name = "QuestLog",
                                    desc = "This setting is display the Japanese translation of QuestLog by default.",
                                    order = 2,
                                    get = function(info)
                                        return self.db.profile.quest.questlog
                                    end,
                                    set = function(info, value)
                                        self.db.profile.quest.questlog = value
                                        WoWJapanizerQuestLog:SetChecked(value)
                                    end,
                                },
                                desc2 = {
                                    type = "description",
                                    name = "This setting can move Japanese translation Window of QuestLog.",
                                    order = 3,
                                },
                                movable = {
                                    type = "toggle",
                                    name = "Enable Movable Window",
                                    desc = "This setting can move Japanese translation Window of QuestLog.",
                                    order = 4,
                                    get = function(info)
                                        return self.db.profile.quest.questlog_movable
                                    end,
                                    set = function(info, value)
                                        self.db.profile.quest.questlog_movable = value
                                        WoWJapanizerQuestLog:SetMovable(value)
                                    end,
                                },
                            },
                        },
                        worldmap = {
                            type = "group",
                            name = "WorldMap",
                            inline = true,
                            order = 3,
                            args = {
                                desc = {
                                    type = "description",
                                    name = "This setting is display the Japanese translation of WorldMap Quest by default.",
                                    order = 1,
                                },
                                worldmap = {
                                    type = "toggle",
                                    name = "WorldMap",
                                    desc = "This setting is display the Japanese translation of WorldMap Quest by default.",
                                    order = 2,
                                    get = function(info)
                                        return self.db.profile.quest.worldmap
                                    end,
                                    set = function(info, value)
                                        self.db.profile.quest.worldmap = value
                                        WoWJapanizerWorldMapQuest:SetChecked(value)
                                    end,
                                },
                            },
                        },
                        gossip = {
                            type = "group",
                            name = "QuestGiver(NPC)",
                            inline = true,
                            order = 4,
                            args = {
                                desc = {
                                    type = "description",
                                    name = "This setting is display the Japanese translation of QuestGiver(NPC).",
                                    order = 1,
                                },
                                gossip = {
                                    type = "toggle",
                                    name = "QuestGiver(NPC)",
                                    desc = "This setting is display the Japanese translation of QuestGiver(NPC).",
                                    order = 2,
                                    get = function(info)
                                        return self.db.profile.quest.gossip
                                    end,
                                    set = function(info, value)
                                        self.db.profile.quest.gossip = value
                                    end,
                                },
                            },
                        },
                    },
                },
                tooltip = {
                    type = "group",
                    name = "WoWJapanizer ToolTip",
                    args = {
                        item = {
                            type = "group",
                            name = "Item ToolTip",
                            inline = true,
                            order = 3,
                            args = {
                                desc = {
                                    type = "description",
                                    name = GetDescription("item", "Item ToolTip"),
                                    order = 1,
                                },
                                tooltip = {
                                    type = "toggle",
                                    name = "Item ToolTip",
                                    order = 2,
                                    get = function(info)
                                        return self.db.profile.item.tooltip
                                    end,
                                    set = function(info, value)
                                        self.db.profile.item.tooltip = value
                                    end,
                                },
                            },
                        },
                        spell = {
                            type = "group",
                            name = "Spell ToolTip",
                            inline = true,
                            order = 4,
                            args = {
                                desc = {
                                    type = "description",
                                    name = GetDescription("spell", "Spell ToolTip"),
                                    order = 1,
                                },
                                tooltip = {
                                    type = "toggle",
                                    name = "Spell ToolTip",
                                    order = 2,
                                    get = function(info)
                                        return self.db.profile.spell.tooltip
                                    end,
                                    set = function(info, value)
                                        self.db.profile.spell.tooltip = value
                                    end,
                                },
                                buff = {
                                    type = "toggle",
                                    name = "Buff ToolTip",
                                    order = 3,
                                    get = function(info)
                                        return self.db.profile.spell.buff
                                    end,
                                    set = function(info, value)
                                        self.db.profile.spell.buff = value
                                    end,
                                },
                            },
                        },
                        achievement = {
                            type = "group",
                            name = "Achievement  |cff9900ff(Beta Test)|r",
                            inline = true,
                            order = 5,
                            args = {
                                desc = {
                                    type = "description",
                                    name = GetDescription("achievement", "Achievement"),
                                    order = 1,
                                },
                                tooltip = {
                                    type = "toggle",
                                    name = "Achievement ToolTip",
                                    order = 2,
                                    get = function(info)
                                        return self.db.profile.achievement.tooltip
                                    end,
                                    set = function(info, value)
                                        self.db.profile.achievement.tooltip = value
                                    end,
                                },
                                advice = {
                                    type = "toggle",
                                    name = "Advice",
                                    order = 2,
                                    get = function(info)
                                        return self.db.profile.achievement.advice
                                    end,
                                    set = function(info, value)
                                        self.db.profile.achievement.advice = value
                                    end,
                                },
                            },
                        },
                    },
                },
                developer = {
                    type = "group",
                    name = "WoWJapanizer Developer",
                    args = {
                        tooltip = {
                            type = "group",
                            name = "Tooltip",
                            inline = true,
                            order = 1,
                            args = {
                                desc = {
                                    type = "description",
                                    name = "Developer mode. ItemID/SpellID is displayed on ToolTip.",
                                    order = 1,
                                },
                                tooltip = {
                                    type = "toggle",
                                    name = "Enable",
                                    order = 2,
                                    get = function(info)
                                        return self.db.profile.developer
                                    end,
                                    set = function(info, value)
                                        self.db.profile.developer = value
                                    end,
                                },
                            },
                        },
                        debugger = {
                            type = "group",
                            name = "Debug Window",
                            inline = true,
                            order = 2,
                            args = {
                                desc = {
                                    type = "description",
                                    name = "Debug Window. /cj debug open Debug Window.",
                                    order = 1,
                                },
                                tooltip = {
                                    type = "toggle",
                                    name = "Enable",
                                    order = 2,
                                    get = function(info)
                                        return self.db.profile.development.debugger
                                    end,
                                    set = function(info, value)
                                        self.db.profile.development.debugger = value
                                    end,
                                },
                            },
                        },
                    },
                },
            },
        }
    end

    self.property = GetProperty()

    LibStub("AceConfig-3.0"):RegisterOptionsTable("WoWJapanizer", GetOptions())

    local dialog =  LibStub("AceConfigDialog-3.0")
    dialog:AddToBlizOptions("WoWJapanizer", nil, nil, "general")
    dialog:AddToBlizOptions("WoWJapanizer", "Quest", "WoWJapanizer", "quest")
    dialog:AddToBlizOptions("WoWJapanizer", "ToolTip", "WoWJapanizer", "tooltip")
    dialog:AddToBlizOptions("WoWJapanizer", "Developer", "WoWJapanizer", "developer")

    self:RegisterChatCommand("cj", "ChatCommand")
    self:RegisterChatCommand("WoWJapanizer", "ChatCommand")
end

function WoWJapanizer:ChatCommand(input)
    if input == 'debug' then
        -- Debug Window --
        if self.db.profile.development.debugger then
            WoWJapanizerDebugFrame:Show()
        end
    else
        self:OpenConfigDialog()
    end
end

function WoWJapanizer:OpenConfigDialog()
    LibStub("AceConfigDialog-3.0"):Open("WoWJapanizer")
end

function WoWJapanizer:DebugLog(args)
    if not self.DEBUG then
        return
    end

    if type(args) == 'string' then
        self:Print(args)
        return
    end

    if type(args) == 'number' then
        self:Print(tostring(args))
        return
    end

    if type(args) == 'table' then
        self:Print("----------")
        for n, v in pairs(args) do
            self:Print(n .. ' : ' .. v)
        end
        self:Print("----------")
        return
    end
end
